import os
import re
import time
import traceback
import threading
import queue
from lambkid import log
from fabric import Connection
from invoke import Responder
from func_timeout import func_set_timeout

class ExecResult(object):
    def __init__(self, output, exit_status_code,stderr=""):
        self.__stdout = output
        self.__exit_status_code = exit_status_code
        self.__stderr=stderr

    def __str__(self):
        return self.__stdout

    def stdout(self):
        return self.__stdout

    def stderr(self):
        return self.__stderr

    @property
    def output(self):
        return self.__stdout

    @property
    def exit_status_code(self):
        return self.__exit_status_code


class SSHClient(object):
    def __init__(self, ip="127.0.0.1", port=22, username="root", password="",connect_timeout=10):
        self.__ip = ip
        self.__port = port
        self.__username = username
        self.__password = password
        self.__connect_timeout = connect_timeout
        self.__ssh = None
        self.__is_active=False
    @property
    def ip(self):
        return self.__ip

    @property
    def port(self):
        return self.__port

    @property
    def username(self):
        return self.__username

    @property
    def password(self):
        return self.__password

    def wait_for_sshable(self, timeout=600):
        count=0
        while True:
            count+=1
            if count>(timeout/10):
                log.error(
                    f" {self.__ip}:{self.__port} | server {self.__ip} can not ssh: Error. err msg is in time {timeout} server cannot ssh.")
                return False
            if not self.__connect():
                log.warning(f" {self.__ip}:{self.__port} | server {self.__ip} can not ssh.")
                time.sleep(10)
                continue
            try:
                output=self.exec("ls /").output
                if "root" in output.strip():
                    return True
            except Exception as e:
                log.warning(
                    f" {self.__ip}:{self.__port} | server {self.__ip} can not ssh: Error. err msg is {str(e)}")
            time.sleep(10)

    @func_set_timeout(3600)
    def _exec(self, cmd):
        log.info(f" {self.__ip}:{self.__port} | begin to run cmd {cmd}...")
        if not self.__is_active:
            self.__connect()
        try:
            rs=self.__ssh.run(cmd)
            if rs.return_code == 0:
                log.info(f" {self.__ip}:{self.__port} | successful to run cmd {cmd}, output is {rs.stdout.strip()}")
            else:
                log.warning(
                    f" {self.__ip}:{self.__port} | run cmd {cmd},exit_status_code is {rs.return_code}, output is {rs.stdout.strip()},")
            new_rs = ExecResult(rs.stdout.strip(), rs.return_code)
        except Exception as e:
            self.__is_active = False
            new_rs = ExecResult(str(e),255)
        return new_rs

    def exec(self, cmd,timeout=1800):
        return self._exec(cmd)

    def exec_interactive(self, cmd,promt_response=[]):
        if not self.__is_active:
            self.__connect()
        response_list=[]
        for elem in promt_response:
            prompt=elem["prompt"]
            response=elem["response"]
            responser = Responder(
                pattern=prompt,
                response=response + '\n',
            )
            response_list.append(responser)
        rs = self.__ssh.run(cmd, pty=True, watchers=response_list)
        new_rs = ExecResult(rs.stdout.strip(),rs.return_code)
        return new_rs


    def scp_to_remote(self, local_path, remote_path):
        log.info(
        f" {self.__ip}:{self.__port} | Begin to copy file from local {local_path} to remote host {remote_path} ...")
        if not self.__is_active:
            self.__connect()
        self.__ssh.put(local_path,remote_path)
        log.info(
            f" {self.__ip}:{self.__port} | Success to copy file from local {local_path} to remote host{remote_path}: OK.")

    def scp_file_to_local(self, remote_path, local_path):
        log.info(
            f" {self.__ip}:{self.__port} | Begin to copy file from remote {remote_path} to local host {local_path} ...")
        if not self.__is_active:
            self.__connect()
        if os.path.isfile(local_path):
            os.system(f"rm -rf {local_path}")
        self.__ssh.get(remote=remote_path,local=local_path)
        log.info(
            f" {self.__ip}:{self.__port} | Success to copy file from remote {remote_path} to local host{local_path}: OK.")

    def __connect(self):
        log.info(f" {self.__ip}:{self.__port} | begin to create ssh connect...")
        try:
            self.__ssh = Connection(host=self.__ip, port=self.__port, user=self.__username, connect_kwargs={"password": self.__password},
                      connect_timeout=self.__connect_timeout)
            log.info(f" {self.__ip}:{self.__port} | successful to create ssh connect: OK.")
            self.__is_active=True
            return True
        except Exception as e:
            log.error(f" {self.__ip}:{self.__port} | fail create ssh connect: Error.err msg is {str(e)}",exc_info=True)
            self.__is_active = False
            return False

    def __del__(self):
        try:
            self.__ssh.close()
        except:
            pass


from lambkid.libs.log import log
from lambkid.libs.log import get_logger
from lambkid.libs.ssh import SSHClient
from lambkid.libs.utils import read_csv,write_csv,get_all_files,read_json,get_all_files_name


import csv
import json
from pathlib import Path
from lambkid import log

def get_all_files(root_dir, recursive=True, suffix_tuple=()):
    """
        return all file path
    :param root_dir:
    :param recursive:
    :param suffix_tuple:
    :return:
    """
    all_files = []
    if Path(root_dir).exists():
        if Path(root_dir).is_dir():
            if recursive:
                for elem in Path(root_dir).glob("**/*"):
                    if Path(elem).is_file():
                        suffix = Path(elem).suffix
                        if not suffix_tuple:
                            all_files.append(elem)
                        else:
                            if suffix in suffix_tuple:
                                all_files.append(elem)
            else:
                for elem in Path(root_dir).iterdir():
                    if Path(elem).is_file():
                        suffix = Path(elem).suffix
                        if not suffix_tuple:
                            all_files.append(elem)
                        else:
                            if suffix in suffix_tuple:
                                all_files.append(elem)
        else:
            all_files.append(root_dir)
    return all_files


def get_all_files_name(root_dir, recursive=True, suffix_tuple=()):
    """
        return all file path
    :param root_dir:
    :param recursive:
    :param suffix_tuple:
    :return:
    """
    all_files = []
    if Path(root_dir).exists():
        if Path(root_dir).is_dir():
            if recursive:
                for elem in Path(root_dir).glob("**/*"):
                    if Path(elem).is_file():
                        suffix = Path(elem).suffix
                        if not suffix_tuple:
                            if elem.name not in all_files:
                                all_files.append(elem.name)
                        else:
                            if suffix in suffix_tuple:
                                if elem.name not in all_files:
                                    all_files.append(elem.name)
            else:
                for elem in Path(root_dir).iterdir():
                    if Path(elem).is_file():
                        suffix = Path(elem).suffix
                        if not suffix_tuple:
                            if elem.name not in all_files:
                                all_files.append(elem.name)
                        else:
                            if suffix in suffix_tuple:
                                if elem.name not in all_files:
                                    all_files.append(elem.name)
        else:
            all_files.append(root_dir)
    return all_files

def write_csv(file_path, head=None, datas=None):
    try:
        with open(file_path, "a+", encoding="utf-8", newline="") as f:
            csv_writer = csv.writer(f)
            if head:
                csv_writer.writerow(head)
            if datas:
                csv_writer.writerows(datas)
    except Exception as e:
        log.error(f"failed to write datas to csv: Error.err msg is {str(e)}")

def read_csv(file_path):
    datas=[]
    with open(file_path, "r", encoding="utf-8", newline="") as f:
        ctx = csv.reader(f)
        for row in ctx:
            row_list=[row[col] for col in range(2)]
            datas.append(row_list)
    return datas

def read_json(file_path):
    if not Path(file_path).exists():
        log.error(f"json file path {file_path} is not exist, please check...")
        return None
    data=dict()
    try:
        with open(file_path, "r", encoding="utf-8") as f:
            data = json.load(f)
    except Exception as e:
        log.error(f"failed to load json file {file_path},err msg is {str(e)}")
    finally:
        return data